/*
 * Software für die PV-Messstation für den Betrieb in der WiFi-Variante
 * Erstellt von: Jan Niklas Rother
 * Zuletzt bearbeitet am 21.02.2022
 */

#include <Wire.h>
#include <WiFi.h>
#include <PubSubClient.h>
#include "SPIFFS.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>


#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define SEALEVELPRESSURE_HPA (1013.25)

#define TIME_TO_SLEEP 5
#define US_IN_S 1000000
#define THRESHOLD 40

// Replace the next variables with your SSID/Password combination
const char* ssid = ""; //SSID as String
const char* password = ""; //Passwort as String

// Add your MQTT Broker IP address, example:
//const char* mqtt_server = "192.168.43.77";
const char* mqtt_server = "broker.mqtt-dashboard.com";

WiFiClient espClient;
PubSubClient client(espClient);

char msg[50];
int value = 0;

Adafruit_BME280 bme; // I2C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

bool deviceConnected = false;
bool oldDeviceConnected = false;

RTC_DATA_ATTR bool firstRun = true;

//int threshold = 40;
bool touch1detected = false;

void gotTouch1(){
 touch1detected = true;
}


 //---------------------------------------------------

void setup() {
    Serial.begin(115200);
    Wire.begin(23, 19);
    Serial.println("Ich bin im Setup");
    setupDisplay();
    setupSensor();
    setupFlash();
    setupInterrupt(); 
    Serial.println("Ich bin fertig mit dem Setup");
}


    String sensordata = "";
    String strSensordataBLE = "";
    char sensordataBLE[40];
    String temp = "";
    String pres = "";
    String humi = "";
    //int counter = 1;
    RTC_DATA_ATTR int counter = 1;
    esp_sleep_wakeup_cause_t wakeup_reason;
    float temperature = 0;
    float humidity = 0;
    float pressure = 0;
    long lastMsg = 0;
  
void loop() {

    wakeup_reason = esp_sleep_get_wakeup_cause();
    
    if (wakeup_reason == 4)
    {
        Serial.println("Timer");
        setupWifi();
        Serial.println("Wifi verbunden");
        client.setServer(mqtt_server, 1883);
        publishData();
        fetchValues();
        writeToFile();
        counter++;
    }
    else if ( wakeup_reason == 5)
    {
        displayDataOnScreen();
        //Serial.println("Touch");
    }
    else
    {
        Serial.println("Irgendeine Scheiße funktioniert schon wieder nicht");
    }
    
    
    Serial.println("sleep...");
    WiFi.disconnect();
    startDeepSleep();


 }

 //---------------------------------------------------


void setupDisplay()
{
    if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))  // Address 0x3D for 128x64
    {
        Serial.println(F("SSD1306 allocation failed"));
        for(;;);
    }
    
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 10);
    display.display();
}

void setupSensor()
{
    bme.begin(0x76);
}

void setupInterrupt()
{
    touchAttachInterrupt(T0, gotTouch1, THRESHOLD); // GPIO4
}

void setupFlash()
{
    if(firstRun)
    {
        firstRun = false;
        if(!SPIFFS.begin(true))
        {
            Serial.println("An Error has occurred while mounting SPIFFS");
            return;
        }
        
        File file = SPIFFS.open("/test.csv", FILE_WRITE);
        
        if(!file)
        {
            Serial.println("There was an error opening the file for writing");
            return;
        }
  
        if(file.println("SAMPLE,TEMPERATURE,PRESSURE,HUMIDITY"))
        {
            Serial.println("File was written");
        }
        else
        {
            Serial.println("File write failed");
        }
        
        file.close();
    }
    else
    {
        SPIFFS.begin();
    }
}

void setupWifi()
{
    delay(10);
//    Serial.println();
//    Serial.print("Connecting to ");
//    Serial.println(ssid);
    WiFi.begin(ssid, password);

    while (WiFi.status() != WL_CONNECTED)
    {
        delay(500);
        Serial.print(WiFi.status());
    }
    if (WiFi.status() == WL_CONNECTED)
    {
        Serial.println("WiFi connected");
    }

//    Serial.println("");
//    Serial.println("WiFi connected");
//    Serial.println("IP address: ");
//    Serial.println(WiFi.localIP());
}

void displayDataOnScreen()
{
    display.clearDisplay();
    display.setCursor(0, 10);
    
    display.print("Temperature: ");
    display.print(bme.readTemperature());
    display.println("C");
  
    display.print("Pressure: ");
    display.print(bme.readPressure()/100.0F);
    display.println("hPA");
  
    display.print("Humidity: ");
    display.print(bme.readHumidity());
    display.println("%");
    
    display.display();
    
    delay(3000);
    clearDisplay();
    touch1detected = false;
}

void clearDisplay()
{
    display.clearDisplay();
    display.display();
}


void fetchValues()
{
    temp = String(bme.readTemperature());
    pres = String(bme.readPressure()/100.0F);
    humi = String(bme.readHumidity());
    sensordata = String(counter) + "," + temp + "," + pres + "," + humi;
}


void writeToFile()
{
    
    File file = SPIFFS.open("/test.csv", FILE_APPEND);

    if(!file.println(sensordata))
    {
        display.clearDisplay();
        display.setCursor(0, 10);
        display.print("File write failed");
    }
    
    display.display();
    file.close();
}


void publishData(){
if (!client.connected()) {
reconnect();
}
//client.loop();
//
//long now = millis();
//Serial.print("lastMsg: ");
//Serial.println(lastMsg);
//Serial.print("now: ");
//Serial.println(now);
//Serial.print("now - lastMsg: ");
//Serial.println(now - lastMsg);
//if (now - lastMsg > 1000) {
//lastMsg = now;
    
temperature = bme.readTemperature();   
    
// Convert the value to a char array
char tempString[8];
dtostrf(temperature, 1, 2, tempString);
Serial.print("Temperature: ");
Serial.println(tempString);
client.publish("ESP32/Temperaturedataread", tempString);

humidity = bme.readHumidity();
    
// Convert the value to a char array
char humString[8];
dtostrf(humidity, 1, 2, humString);
Serial.print("Humidity: ");
Serial.println(humString);
client.publish("ESP32/Humiditydataread", humString);

pressure = bme.readPressure()/100;
    
// Convert the value to a char array
char preString[8];
dtostrf(pressure, 3, 2, preString);
Serial.print("Pressure: ");
Serial.println(preString);
client.publish("ESP32/Pressuredataread", preString);
delay(100);
//}
}

void reconnect()
{
    // Loop until we're reconnected
    while (!client.connected())
    {
        Serial.print("Attempting MQTT connection...");
        // Attempt to connect
        if (client.connect("ESP32"))
        {
            Serial.println("connected");
        } 
        else
        {
            Serial.print("failed, rc=");
            Serial.print(client.state());
            Serial.println(" try again in 5 seconds");
            // Wait 5 seconds before retrying
            delay(5000);
        }
    }
}

void callback(char* topic, byte* message, unsigned int length)
{
    Serial.print("Message arrived on topic: ");
    Serial.print(topic);
    Serial.print(". Message: ");
    String messageTemp;
  
    for (int i = 0; i < length; i++)
    {
        Serial.print((char)message[i]);
        messageTemp += (char)message[i];
    }
}

void startDeepSleep()
{
    esp_sleep_enable_timer_wakeup(TIME_TO_SLEEP * US_IN_S);
    esp_sleep_enable_touchpad_wakeup();
    esp_deep_sleep_start();
}
